@@ -1,3 +1,4 @@
+import fileinput
 import os
 import shutil
 import unittest
@@ -7,6 +8,7 @@ try:
 except ImportError:
     from io import StringIO
 from mock import patch, MagicMock, call
+from six import assertRaisesRegex
 from txclib.commands import _set_source_file, _set_translation, cmd_pull, \
     cmd_init, cmd_config, cmd_status, cmd_help, UnInitializedError
 from txclib.cmdline import main
@@ -80,10 +82,9 @@ class TestInitCommand(unittest.TestCase):
     def test_init(self):
         argv = []
         config_text = "[main]\nhost = https://www.transifex.com\n\n"
-        with patch('txclib.commands.project.Project') as project_mock:
+        with patch('txclib.commands.project.Project'):
             with patch('txclib.commands.cmd_config') as set_mock:
                 cmd_init(argv, '')
-                project_mock.assert_called()
                 set_mock.assert_called_once_with([], os.getcwd())
         self.assertTrue(os.path.exists('./.tx'))
         self.assertTrue(os.path.exists('./.tx/config'))
@@ -91,10 +92,9 @@ class TestInitCommand(unittest.TestCase):
 
     def test_init_skipsetup(self):
         argv = ['--skipsetup']
-        with patch('txclib.commands.project.Project') as project_mock:
+        with patch('txclib.commands.project.Project'):
             with patch('txclib.commands.cmd_config') as set_mock:
                 cmd_init(argv, '')
-                project_mock.assert_called()
                 self.assertEqual(set_mock.call_count, 0)
         self.assertTrue(os.path.exists('./.tx'))
         self.assertTrue(os.path.exists('./.tx/config'))
@@ -117,10 +117,9 @@ class TestInitCommand(unittest.TestCase):
         open('./.tx/config', 'a').close()
         argv = []
         confirm_mock.return_value = True
-        with patch('txclib.commands.project.Project') as project_mock:
+        with patch('txclib.commands.project.Project'):
             with patch('txclib.commands.cmd_config') as set_mock:
                 cmd_init(argv, '')
-                project_mock.assert_called()
                 set_mock.assert_called()
         self.assertTrue(os.path.exists('./.tx'))
         self.assertEqual(confirm_mock.call_count, 1)
@@ -128,10 +127,9 @@ class TestInitCommand(unittest.TestCase):
     def test_init_force_save(self):
         os.mkdir('./.tx')
         argv = ['--force-save']
-        with patch('txclib.commands.project.Project') as project_mock:
+        with patch('txclib.commands.project.Project'):
             with patch('txclib.commands.cmd_config') as set_mock:
                 cmd_init(argv, '')
-                project_mock.assert_called()
                 set_mock.assert_called()
         self.assertTrue(os.path.exists('./.tx'))
         self.assertTrue(os.path.exists('./.tx/config'))
@@ -164,7 +162,8 @@ class TestPullCommand(unittest.TestCase):
         pull_call = call(
             branch='a-branch', fetchall=False, fetchsource=False,
             force=False, languages=[], minimum_perc=None, mode=None,
-            overwrite=True, pseudo=False, resources=[], skip=False, xliff=False
+            overwrite=True, pseudo=False, resources=[], skip=False,
+            xliff=False, parallel=False, no_interactive=False
         )
         pr_instance.pull.assert_has_calls([pull_call])
 
@@ -182,10 +181,26 @@ class TestPullCommand(unittest.TestCase):
         pull_call = call(
             branch='somebranch', fetchall=False, fetchsource=False,
             force=False, languages=[], minimum_perc=None, mode=None,
-            overwrite=True, pseudo=False, resources=[], skip=False, xliff=False
+            overwrite=True, pseudo=False, resources=[], skip=False,
+            xliff=False, parallel=False, no_interactive=False
         )
         pr_instance.pull.assert_has_calls([pull_call])
 
+    @patch('txclib.commands.project')
+    def test_pull_with_no_interactive(self, project_mock):
+        pr_instance = MagicMock()
+        pr_instance.pull.return_value = True
+        project_mock.Project.return_value = pr_instance
+        cmd_pull(['--no-interactive'], '.')
+        pull_call = call(
+            fetchall=False, force=False, minimum_perc=None,
+            skip=False, no_interactive=True, resources=[], pseudo=False,
+            languages=[], fetchsource=False, mode=None, branch=None,
+            xliff=False, parallel=False, overwrite=True
+        )
+        self.assertEqual(pr_instance.pull.call_count, 1)
+        pr_instance.pull.assert_has_calls([pull_call])
+
 
 class TestConfigCommand(unittest.TestCase):
 
@@ -195,7 +210,9 @@ class TestConfigCommand(unittest.TestCase):
         os.mkdir('.tx')
         self.path_to_tx = os.getcwd()
         self.config_file = '.tx/config'
-        open(self.config_file, "w").write('[main]\nhost = https://foo.var\n')
+        self.config_fd = open(self.config_file, "w")
+        self.config_fd.write("[main]\nhost = https://foo.var\n")
+        self.config_fd.close()
 
     def tearDown(self, *args, **kwargs):
         shutil.rmtree('.tx', ignore_errors=False, onerror=None)
@@ -235,7 +252,8 @@ class TestConfigCommand(unittest.TestCase):
             self.assertEqual(config.read(), expected)
 
     def test_auto_locale_no_expression(self):
-        with self.assertRaises(SystemExit):
+        error_msg = "You need to specify an expression"
+        with assertRaisesRegex(self, Exception, error_msg):
             args = [MAPPING, "-r", "project1.resource1",
                     '--source-language', 'en']
             cmd_config(args, self.path_to_tx)
@@ -389,6 +407,53 @@ class TestConfigCommand(unittest.TestCase):
         with open(self.config_file) as config:
             self.assertEqual(config.read(), expected)
 
+    def test_invalid_expression(self):
+        args = [MAPPINGBULK, "-p", "test-project", "--source-file-dir",
+                "translations", "--source-language", "en", "-t", "TXT",
+                "--file-extension", ".txt", "--execute", "--expression",
+                "expression/{filename}{extension}"]
+        with self.assertRaises(SystemExit):
+            cmd_config(args, self.path_to_tx)
+
+    def test_invalid_expression_status(self):
+        """
+        Test how the client handles invalid expressions in file_filter
+        """
+        valid_expression = "test_expressions/<lang>/test.txt"
+        invalid_expression = "test_expressions/{filename}{ext}"
+        args = [MAPPINGBULK, "-p", "test-project", "--source-file-dir",
+                "test_expressions/en", "--source-language", "en", "-t", "TXT",
+                "--file-extension", ".txt", "--execute", "--expression",
+                valid_expression]
+        # Configure the client using a valid expression
+        cmd_config(args, self.path_to_tx)
+
+        # A trick to capture the standard output
+        sys_stdout = sys.stdout
+        out = StringIO()
+        sys.stdout = out
+        # Check for the expected status using the valid expression
+        cmd_status([], None)
+        output = out.getvalue().strip()
+        sys.stdout = sys_stdout
+        self.assertTrue("test_expressions/en/test.txt" in output)
+        self.assertTrue("test_expressions/es/test.txt" in output)
+
+        # Change the configuration file "by hand" and replace the file_filter
+        # with an invalid expression
+        for line in fileinput.input(self.config_file, inplace=True):
+            print(line.replace(valid_expression, invalid_expression))
+
+        sys_stdout = sys.stdout
+        out = StringIO()
+        sys.stdout = out
+        cmd_status([], None)
+        output = out.getvalue().strip()
+        sys.stdout = sys_stdout
+        # Check that the Spanish translation file in not tracked
+        self.assertTrue("test_expressions/en/test.txt" in output)
+        self.assertFalse("test_expressions/es/test.txt" in output)
+
 
 class TestMainCommand(unittest.TestCase):
     def test_call_tx_with_no_command(self):
