@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import os
 import unittest
 import itertools
 try:
@@ -12,10 +13,10 @@ except ImportError:
     import ConfigParser as configparser
 
 from functools import wraps
-from mock import Mock, patch
+from mock import Mock, patch, mock_open
 from collections import namedtuple
 from os.path import dirname
-from sys import modules
+from sys import modules, version_info
 
 from txclib.exceptions import AuthenticationError, TXConnectionError
 from txclib.project import (Project, DEFAULT_PULL_URL)
@@ -93,10 +94,65 @@ class TestProject(unittest.TestCase):
         username, password = p.getset_host_credentials('test')
         self.assertEqual(username, 'api')
         self.assertEqual(password, dummy_token)
-        self.assertEqual(p.txrc.set.call_count, 4)
+        self.assertEqual(p.txrc.set.call_count, 2)
         self.assertEqual(m_input.call_count, 1)
         p.save.assert_called()
 
+    @patch('txclib.project.input')
+    @patch('txclib.config.configparser')
+    @patch.dict(os.environ, {'TX_TOKEN': 'environment_value'})
+    def test_getset_host_credentials_env_variable(
+            self, m_parser, m_input):
+        p = Project(init=False)
+        p.txrc = m_parser
+        p.save = Mock()
+        p.txrc_file = '/tmp'
+        p.txrc.has_section.side_effect = [False]
+        username, password = p.getset_host_credentials('test')
+        self.assertEqual(username, 'api')
+        self.assertEqual(password, 'environment_value')
+        self.assertEqual(p.txrc.set.call_count, 2)
+        # no input will be asked, password will be used by environment variable
+        self.assertEqual(m_input.call_count, 0)
+        p.save.assert_called()
+
+    @patch('txclib.project.input')
+    @patch('txclib.config.configparser')
+    @patch.dict(os.environ, {'TX_TOKEN': 'environment_value'})
+    def test_getset_host_credentials_env_variable_first_time(
+            self, m_parser, m_input):
+        p = Project(init=False)
+        p.txrc = m_parser
+        p.save = Mock()
+        p.txrc_file = '/tmp'
+        p.txrc.has_section.side_effect = [False]
+        username, password = p.getset_host_credentials('test')
+        self.assertEqual(username, 'api')
+        self.assertEqual(password, 'environment_value')
+        # ensure that we have set host in the txrc_file, even though TX_TOKEN
+        # exists
+        self.assertEqual(p.txrc.set.call_count, 2)
+        p.save.assert_called()
+
+    @patch('txclib.config.configparser')
+    @patch("txclib.project.logger")
+    @patch.dict(os.environ, {'TX_TOKEN': 'environment_value'})
+    def test_getset_host_credentials_both_token_and_env(
+            self, m_logger, m_parser):
+        p = Project(init=False)
+        p.txrc = m_parser
+        p.save = Mock()
+        p.txrc_file = '/tmp'
+        p.txrc.has_section.side_effect = [False]
+        username, password = p.getset_host_credentials('test', token='demo')
+        self.assertEqual(username, 'api')
+        self.assertEqual(password, 'environment_value')
+        # ensure that we did not make additional calls to set the token in the
+        # txrc file
+        self.assertEqual(p.txrc.set.call_count, 2)
+        p.save.assert_called()
+        self.assertEqual(m_logger.warning.call_count, 1)
+
     @patch('txclib.project.utils.confirm')
     @patch('txclib.config.configparser')
     def test_getset_host_credentials_update_transifexrc(
@@ -143,7 +199,7 @@ class TestProject(unittest.TestCase):
                                                        token=dummy_token)
         self.assertEqual(username, 'api')
         self.assertEqual(password, dummy_token)
-        self.assertEqual(p.txrc.set.call_count, 4)
+        self.assertEqual(p.txrc.set.call_count, 2)
         self.assertEqual(m_input.call_count, 1)
         p.save.assert_called()
 
@@ -509,7 +565,7 @@ class TestProjectPull(unittest.TestCase):
         self.lang_map = Flipdict()
 
     def test_new_translations(self):
-        """Test finding new transaltions to add."""
+        """Test finding new translations to add."""
         with patch.object(self.p, 'do_url_request') as resource_mock:
             resource_mock.return_value = json.dumps(self.details), "utf-8"
             new_trans = self.p._new_translations_to_add
@@ -627,7 +683,7 @@ class TestProjectPull(unittest.TestCase):
             self.assertEqual(new, set([]))
 
     def test_in_combination_with_force_option(self):
-        """Test the minumum-perc option along with -f."""
+        """Test the minimum-perc option along with -f."""
         with patch.object(self.p, 'get_resource_option') as mock:
             mock.return_value = 70
 
@@ -686,7 +742,8 @@ class TestProjectPull(unittest.TestCase):
                 # Create fake https response
                 def encode_args(*args, **kwargs):
                     struct = namedtuple("response", "data status close")
-                    return struct(status=401, data="mock_response", close=Mock())
+                    return struct(status=401, data="mock_response",
+                                  close=Mock())
                 mock_determine_charset.return_value = "utf-8"
                 mock_encode_args.return_value = encode_args
 
@@ -792,6 +849,27 @@ class TestProjectPull(unittest.TestCase):
         with self.assertRaises(TXConnectionError):
             project.push()
 
+    @fixture_mocked_project
+    @patch('txclib.utils.queue_request')
+    @patch('txclib.utils.make_request')
+    @patch('txclib.project.Project._resource_exists')
+    def test_push_async(self, mock_exists, mock_request, mock_queue, **kwargs):
+        mock_queue.return_value = None
+        mock_request.return_value = ('{"i18n_type": "PO"}', '')
+        mock_exists.return_value = True
+        project = kwargs['mock_project']
+        patch_open = ("builtins.open" if version_info.major > 2
+                      else "__builtin__.open")
+        with patch(patch_open, mock_open(read_data="")):
+            project.push(source=True, parallel=True)
+            self.assertEqual(mock_queue.call_count, 1)
+
+            project.push(translations=True, parallel=True)
+            self.assertEqual(mock_queue.call_count, 1)
+
+            project.pull(parallel=True)
+            self.assertEqual(mock_queue.call_count, 1)
+
 
 class TestFormats(unittest.TestCase):
     """Tests for the supported formats."""
