@@ -1,21 +1,21 @@
 # -*- coding: utf-8 -*-
-"""In this file we have all the top level commands for the transifex client.
-Since we're using a way to automatically list them and execute them, when
-adding code to this file you must take care of the following:
+"""
+In this file we have all the top level commands for the Transifex client. Since
+we're using a way to automatically list them and execute them, when adding code
+to this file you must take care of the following:
  * Added functions must begin with 'cmd_' followed by the actual name of the
-   command being used in the command line (eg cmd_init)
+   command being used in the command line (e.g. cmd_init)
  * The description for each function that we display to the user is read from
    the func_doc attribute which reads the doc string. So, when adding
-   docstring to a new function make sure you add an oneliner which is
+   docstring to a new function make sure you add an one-liner which is
    descriptive and is meant to be seen by the user.
  * When including libraries, it's best if you include modules instead of
    functions because that way our function resolution will work faster and the
    chances of overlapping are minimal
  * All functions should use the OptionParser and should have a usage and
-   descripition field.
+   description field.
 """
 import os
-import re
 import sys
 try:
     import configparser
@@ -36,7 +36,7 @@ from txclib import messages
 
 
 def cmd_init(argv, path_to_tx):
-    """Initialize a new transifex project."""
+    """Initialize a new Transifex project."""
     parser = init_parser()
     options = parser.parse_args(argv)
     path_to_tx = options.path_to_tx or os.getcwd()
@@ -76,38 +76,40 @@ def cmd_init(argv, path_to_tx):
         config.write(fh)
         fh.close()
 
-    prj = project.Project(path_to_tx)
-    prj.getset_host_credentials(transifex_host, username=options.user,
-                                password=options.password,
-                                token=options.token, force=options.save,
-                                no_interactive=options.no_interactive)
-    prj.save()
-
     if not options.skipsetup and not options.no_interactive:
         logger.info(messages.running_tx_set)
         cmd_config([], path_to_tx)
     else:
+        prj = project.Project(path_to_tx)
+        prj.getset_host_credentials(transifex_host, username=options.user,
+                                    password=options.password,
+                                    token=options.token, force=options.save,
+                                    no_interactive=options.no_interactive)
+        prj.save()
         logger.info("Done.")
 
 
 def cmd_set(argv, path_to_tx):
-    """Add local or remote files under transifex. Warning: \
-This command will be deprecated in a future release of the \
-client. You should use the `tx config` command."""
+    """
+    Add local or remote files under Transifex. Warning: This command will be
+    deprecated in a future release of the client. You should use the
+    `tx config` command.
+    """
     cmd_config(argv, path_to_tx, is_legacy=True)
 
 
 def cmd_config(argv, path_to_tx, is_legacy=False):
-    """Add local or remote files under transifex"""
+    """Add local or remote files under Transifex"""
     from_wizard = False
     if len(argv) == 0:
-        # since interactive wizard should be equivalant to auto-local
+        # since interactive wizard should be equivalent to auto-local
         # subcommand there are some default options that need to be set
         default_options = {
             'execute': True,
             'subcommand': MAPPING,
             'minimum_perc': 0,
             'mode': None,
+            'expression_legacy': None,
         }
         try:
             # Run the wizard and configure parse with the wizard inputs
@@ -189,7 +191,7 @@ def bare_set(path_to_tx, options):
 
 
 def subcommand_mapping(path_to_tx, options, from_wizard=False):
-    expression = options.expression
+    expression = options.expression or options.expression_legacy
     _auto_local(path_to_tx, options.resource,
                 source_language=options.source_language,
                 expression=expression, source_file=options.source_file,
@@ -220,7 +222,7 @@ def _print_instructions(resource, path_to_tx):
 
 
 def subcommand_mapping_bulk(path_to_tx, options, **kwargs):
-    """Add local files for multiple resources under transifex"""
+    """Add local files for multiple resources under Transifex"""
 
     if not options.file_extension.startswith('.'):
         file_extension = '.{}'.format(options.file_extension)
@@ -274,8 +276,9 @@ def _auto_local(path_to_tx, resource, source_language, expression,
     curpath = os.path.abspath(os.curdir)
 
     # Force expr to be a valid regex expr (escaped) but keep <lang> intact
-    expr_re = utils.regex_from_filefilter(expression, curpath)
-    expr_rec = re.compile(expr_re)
+    if not expression:
+        raise Exception("You need to specify an expression to define where "
+                        "translation files should be saved.")
 
     if not execute:
         logger.info("Only printing the commands which will be run if the "
@@ -284,14 +287,11 @@ def _auto_local(path_to_tx, resource, source_language, expression,
     # First, let's construct a dictionary of all matching files.
     # Note: Only the last matching file of a language will be stored.
     translation_files = {}
-    for f_path in utils.files_in_project(curpath):
-        match = expr_rec.match(posix_path(f_path))
-        if match:
-            lang = match.group(1)
-            if lang == source_language and not source_file:
-                source_file = f_path
-            else:
-                translation_files[lang] = f_path
+    for f_path, lang in utils.get_project_files(curpath, expression):
+        if lang == source_language and not source_file:
+            source_file = f_path
+        else:
+            translation_files[lang] = f_path
 
     if not source_file:
         raise Exception("Could not find a source language file. Please run "
@@ -324,7 +324,7 @@ def _auto_local(path_to_tx, resource, source_language, expression,
     if execute:
         logger.info("Updating file expression for resource %s ( %s )." % (
                     resource, expression))
-        # Eval file_filter relative to root dir
+        # Evaluate file_filter relative to root dir
         file_filter = posix_path(
             os.path.relpath(os.path.join(curpath, expression), path_to_tx)
         )
@@ -413,6 +413,7 @@ def cmd_push(argv, path_to_tx):
     resources = parse_csv_option(options.resources)
     skip = options.skip_errors
     xliff = options.xliff
+    parallel = options.parallel
     prj = project.Project(path_to_tx)
     if not (options.push_source or options.push_translations):
         parser.error("You need to specify at least one of the -s|--source, "
@@ -424,7 +425,7 @@ def cmd_push(argv, path_to_tx):
         skip=skip, source=options.push_source,
         translations=options.push_translations,
         no_interactive=options.no_interactive,
-        xliff=xliff, branch=branch
+        xliff=xliff, branch=branch, parallel=parallel
     )
     logger.info("Done.")
 
@@ -441,6 +442,7 @@ def cmd_pull(argv, path_to_tx):
     pseudo = options.pseudo
     # Should we download as xliff?
     xliff = options.xliff
+    parallel = options.parallel
     skip = options.skip_errors
     minimum_perc = options.minimum_perc or None
 
@@ -453,7 +455,8 @@ def cmd_pull(argv, path_to_tx):
         languages=languages, resources=resources, overwrite=options.overwrite,
         fetchall=options.fetchall, fetchsource=options.fetchsource,
         force=options.force, skip=skip, minimum_perc=minimum_perc,
-        mode=options.mode, pseudo=pseudo, xliff=xliff, branch=branch
+        mode=options.mode, pseudo=pseudo, xliff=xliff, branch=branch,
+        parallel=parallel, no_interactive=options.no_interactive,
     )
     logger.info("Done.")
 
@@ -590,7 +593,7 @@ def cmd_help(argv, path_to_tx):
     # or print summary of all commands
 
     # the code below will only be executed if the KeyError exception is thrown
-    # becuase in all other cases the function called with --help will exit
+    # because in all other cases the function called with --help will exit
     # instead of return here
     keys = list(fns.keys())
     keys.sort()
@@ -620,7 +623,7 @@ def _go_to_dir(path):
     argument.
 
     Args:
-        path: The path to chdor to.
+        path: The path to chdir to.
     Raises:
         UnInitializedError, in case the directory has not been initialized.
     """
@@ -651,7 +654,7 @@ def _set_type(resource, value, path_to_tx):
 
 
 def _set_project_option(resource, name, value, path_to_tx, func_name):
-    """Save the option to the project config file."""
+    """Save the option to the project configuration file."""
     if value is None:
         return
     if not resource:
