@@ -99,11 +99,13 @@ def help_parser():
 
 def init_parser():
     """Return the command-line parser for the init command."""
-    description = "This command initializes a new project for use with "\
-        "Transifex. It is recommended to execute this command in the "\
-        "top level directory of your project so that you can include "\
-        "all files under it in transifex. If no path is provided, the "\
-        "current working dir will be used."
+    description = (
+        "This command initializes a new project for use with Transifex. It "
+        "is recommended to execute this command in the top level directory "
+        "of your project so that you can include all files under it in "
+        "Transifex. If no path is provided, the current working directory"
+        "will be used."
+    )
     parser = ArgumentParser(description=description)
     parser.add_argument("--host", action="store", dest="host", default=None,
                         help="Specify a default Transifex host.")
@@ -196,6 +198,11 @@ def pull_parser():
     parser.add_argument("-x", "--xliff", action="store_true", dest="xliff",
                         default=False, help="Apply this option to download "
                         "file as xliff.")
+    parser.add_argument("--parallel", action="store_true", default=False,
+                        help="perform push/pull requests in parallel")
+    parser.add_argument("--no-interactive", action="store_true",
+                        dest="no_interactive", default=False,
+                        help="Don't require user input.")
     return parser
 
 
@@ -243,6 +250,8 @@ def push_parser():
         help=("Pull for a specific branch. Default is current"
               "branch if exists.")
     )
+    parser.add_argument("--parallel", action="store_true", default=False,
+                        help="perform push/pull requests in parallel")
     return parser
 
 
@@ -388,11 +397,14 @@ def set_parser(subparser=False, is_legacy=False):
         "--execute", action="store_true", dest="execute", default=False,
         help="Execute commands.")
     auto_local_parser.add_argument(
-        "expression", action="store",
-        help="A path expression pointing to the location where translation "
-             "files for the associated source file are/will be saved. Use "
-             "<lang> as a wildcard for the language code, "
-             "e.g. translations/<lang>/test.txt."
+        "--expression", action="store",
+        help=("Expression defining where translation files should be saved. "
+              "Default value is: "
+              "'locale/<lang>/{filepath}/{filename}{extension}'")
+    )
+    auto_local_parser.add_argument(
+        "expression_legacy", action="store", nargs="?",
+        help="DEPRECATED: Use --expression instead."
     )
 
     # auto-remote subparser
