@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 import os
-import re
 import fnmatch
 import datetime
 import time
@@ -11,7 +10,7 @@ import six
 try:
     import urlparse
     from urllib import urlencode
-except:  # For Python 3
+except ImportError:  # For Python 3
     import urllib.parse as urlparse
     from urllib.parse import urlencode
 try:
@@ -35,19 +34,20 @@ from txclib.exceptions import (
 from txclib.urls import API_URLS
 from txclib.config import Flipdict, CERT_REQUIRED
 from txclib.log import logger
-from txclib.paths import posix_path, native_path, posix_sep
-from txclib.utils import ProjectNotInit
+from txclib.paths import native_path, posix_sep
+from txclib.utils import ProjectNotInit, perform_parallel_requests
 
 
 DEFAULT_PULL_URL = 'pull_file'
 PULL_MODE_URL_NAME = "pull_{mode}_file"
 
 DEFAULT_API_HOSTNAME = "https://api.transifex.com"
+API_USERNAME = api.Api.USERNAME
 
 
 class Project(object):
-    """Represents an association between the local and
-    remote project instances.
+    """
+    Represent an association between the local and remote project instances.
     """
 
     SKIP_DECODE_I18N_TYPES = ['DOCX', 'XLSX']
@@ -109,6 +109,29 @@ class Project(object):
 
         save = False
         config_username, config_password = None, None
+        if 'TX_TOKEN' in os.environ:
+            # User api info from environment variable
+            password = os.environ['TX_TOKEN']
+            username = API_USERNAME
+            """
+            We need to check if hostname info exists in the .transifexrc file
+            If not, it means that this is the first time this function runs, so
+            we need to create its data.
+            """
+            save = self._add_host_to_config_file(host)
+            if save:
+                logger.info("Using TX_TOKEN environment variable. Credentials "
+                            "won't be saved to .transifexrc.")
+                self.save()
+            if token:
+                # Both env and argument given, inform the user
+                logger.warning(
+                    "There is a token in the TX_TOKEN env variable and the "
+                    "--token argument. The latter will be ignored and no "
+                    "credentials will be saved to the .transifexrc file."
+                )
+            return username, password
+
         try:
             config_username = self.txrc.get(host, 'username')
             config_password = self.txrc.get(host, 'password')
@@ -117,13 +140,13 @@ class Project(object):
 
         if token:
             password = token
-            username = 'api'
+            username = API_USERNAME
 
         if not (username and password) and not \
                (config_username and config_password):
-            token = self._token_prompt(host)
-            username = 'api'
-            password = token
+            username = API_USERNAME
+            if not no_interactive:
+                password = self._token_prompt(host)
             save = True
         elif config_username and config_password:
             if username == config_username and password == config_password:
@@ -141,22 +164,34 @@ class Project(object):
         # api we can only use a token and not a password so we do an extra
         # validation and prompt the use for a token if the validation fails
         if only_token and not self.validate_credentials(username, password):
-            logger.info("You need a valid api token to proceed")
-            username = 'api'
+            logger.info("You need a valid API token to proceed")
+            username = API_USERNAME
             password = self._token_prompt(host)
             save = True
 
         if save:
-            logger.info("Updating %s file..." % self.txrc_file)
-            if not self.txrc.has_section(host):
-                self.txrc.add_section(host)
+            self._add_host_to_config_file(host)
             self.txrc.set(host, 'username', username)
             self.txrc.set(host, 'password', password)
+            self.save()
+        return username, password
+
+    def _add_host_to_config_file(self, host):
+        """Check if a given host exists in .transifexrc, and if not it
+        updates the file with the appropriate info.
+
+        :param str host: The hostname to search for and add if needed.
+        :return: True if adding a new host, nothing otherwise.
+        :rtype: bool
+        """
+        if not self.txrc.has_section(host):
+            logger.info("Updating %s file..." % self.txrc_file)
+            self.txrc.add_section(host)
             self.txrc.set(host, 'hostname', host)
             self.txrc.set(host, 'api_hostname',
                           utils.DEFAULT_HOSTNAMES['api_hostname'])
-            self.save()
-        return username, password
+            return True
+        return False
 
     def _token_prompt(self, host):
         token = None
@@ -164,14 +199,14 @@ class Project(object):
         while not token:
             token = input(utils.color_text(messages.token_msg, COLOR))
             print(utils.color_text("Verifying token...", "YELLOW"))
-            if not self.validate_credentials('api', token, host):
+            if not self.validate_credentials(API_USERNAME, token, host):
                 logger.info(messages.token_validation_failed)
                 token = None
         return token
 
     def set_remote_resource(self, resource, source_lang, i18n_type, host,
                             file_filter=None):
-        """Method to handle the add/conf of a remote resource."""
+        """Method to handle the addition/configuration of a remote resource."""
         if file_filter is None:
             file_filter = self.FILE_FILTER
 
@@ -195,7 +230,7 @@ class Project(object):
             self.config.set(resource, 'host', host)
 
     def get_resource_host(self, resource):
-        """Returns the host that the resource is configured to use.
+        """Return the host that the resource is configured to use.
         If there is no such option we return the default one.
         """
         return self.config.get('main', 'host')
@@ -253,7 +288,7 @@ class Project(object):
     def get_resource_files(self, resource, xliff=False):
         """Get a dict for all files assigned to a resource.
         First we calculate the files matching the file expression and
-        then we apply all translation excpetions.
+        then we apply all translation exceptions.
         The resulting dict will be in this format:
 
         { 'en': 'path/foo/en/bar.po',
@@ -273,21 +308,12 @@ class Project(object):
                 file_filter += '.xlf'
             source_lang = self.config.get(resource, "source_lang")
             source_file = self.get_source_file(resource)
-            expr_re = utils.regex_from_filefilter(file_filter, self.root)
-            expr_rec = re.compile(expr_re)
-            for f_path in utils.files_in_project(self.root):
-                match = expr_rec.match(posix_path(f_path))
-                if match:
-                    try:
-                        lang = match.group(1)
-                    except IndexError:
-                        msg = ("file_filter {} does not contain '<lang>' "
-                               "expresion".format(file_filter))
-                        raise MalformedConfigFile(msg)
-                    if lang != source_lang:
-                        f_path = os.path.relpath(f_path, self.root)
-                        if f_path != source_file:
-                            tr_files.update({lang: f_path})
+            matched_files = utils.get_project_files(self.root, file_filter)
+            for f_path, lang in matched_files:
+                if lang != source_lang:
+                    f_path = os.path.relpath(f_path, self.root)
+                    if f_path != source_file:
+                        tr_files.update({lang: f_path})
 
             for (name, value) in self.config.items(resource):
                 if name.startswith("trans."):
@@ -324,7 +350,7 @@ class Project(object):
         return None
 
     def get_resource_list(self, project=None):
-        """Parse config file and return tuples with the following format
+        """Parse configuration file and return tuples with the following format
 
         [ (project_slug, resource_slug), (..., ...)]
         """
@@ -341,8 +367,9 @@ class Project(object):
         return resource_list
 
     def save(self):
-        """Store the config dictionary
-        in the .tx/config file of the project.
+        """
+        Store the configuration dictionary in the .tx/config file of the
+        project.
         """
         utils.save_tx_config(self.config_file, self.config)
         utils.save_txrc_file(self.txrc_file, self.txrc)
@@ -364,10 +391,14 @@ class Project(object):
         return '{branch}--{resource}'.format(branch=slugify(branch),
                                              resource=resource_slug)
 
-    def pull(self, languages=[], resources=[], overwrite=True, fetchall=False,
-             fetchsource=False, force=False, skip=False, minimum_perc=0,
-             mode=None, pseudo=False, xliff=False, branch=None):
-        """Pull all translations file from transifex server."""
+    def pull(self, languages=None, resources=None, overwrite=True,
+             fetchall=False, fetchsource=False, force=False, skip=False,
+             minimum_perc=0, mode=None, pseudo=False, xliff=False, branch=None,
+             parallel=False, no_interactive=False):
+        """Pull all translations file from Transifex server."""
+        languages = languages or []
+        resources = resources or []
+
         self.minimum_perc = minimum_perc
         resource_list = self.get_chosen_resources(resources)
         skip_decode = False
@@ -394,7 +425,9 @@ class Project(object):
             logger.debug("URL data are: %s" % self.url_info)
 
             try:
-                stats = self._get_stats_for_resource()
+                stats = self._get_stats_for_resource(
+                    no_interactive=no_interactive
+                )
                 details_response, _ = self.do_url_request('resource_details')
             except Exception as e:
                 if isinstance(e, HttpNotAuthorized):
@@ -461,8 +494,9 @@ class Project(object):
                     new_translations.add(slang)
 
             if pull_languages:
-                logger.debug("Pulling languages for: %s" % pull_languages)
-                msg = "Pulling translations for resource %s (source: %s)"
+                prefix = "Queueing" if parallel else "Pulling"
+                logger.debug("%s languages for: %s" % (prefix, pull_languages))
+                msg = prefix + " translations for resource %s (source: %s)"
                 if branch:
                     msg += " for branch {}".format(branch)
                 if xliff:
@@ -505,29 +539,24 @@ class Project(object):
                     continue
 
                 if xliff:
-                    local_file += '.xlf'
+                    local_file += ".xlf"
 
                 if not overwrite:
-                    local_file = ("%s.new" % local_file)
+                    local_file += ".new"
+
                 logger.warning(
                     " -> %s: %s" % (utils.color_text(remote_lang, "RED"),
                                     local_file)
                 )
-                try:
-                    r, charset = self.do_url_request(
-                        url, language=remote_lang, skip_decode=skip_decode,
-                        params=params
-                    )
-                except Exception as e:
-                    if isinstance(e, SSLError) or not skip:
-                        raise
-                    else:
-                        logger.error(e)
-                        continue
-                self._save_file(local_file, charset, r)
+                self.do_url_request(
+                    url, language=remote_lang, skip_decode=skip_decode,
+                    params=params, parallel=parallel, callback=self._save_file,
+                    callback_args={"local_file": local_file}
+                )
 
             if new_translations:
-                msg = "Pulling new translations for resource %s (source: %s)"
+                prefix = "Queueing" if parallel else "Pulling"
+                msg = prefix + " new translations for resource %s (source: %s)"
                 logger.info(msg % (resource, sfile))
                 for lang in new_translations:
                     if lang in list(lang_map.keys()):
@@ -552,6 +581,9 @@ class Project(object):
                                          '%s_translation' % local_lang,
                                          os.curdir))
 
+                    if xliff:
+                        local_file += ".xlf"
+
                     if lang != slang:
                         satisfies_min = self._satisfies_min_translated(
                             stats[remote_lang], mode
@@ -565,19 +597,25 @@ class Project(object):
                                         local_file)
                     )
 
-                    r, charset = self.do_url_request(
+                    self.do_url_request(
                         url, language=remote_lang, skip_decode=skip_decode,
-                        params=params
+                        params=params, parallel=parallel,
+                        callback=self._save_file,
+                        callback_args={"local_file": local_file}
                     )
-                    if xliff:
-                        local_file += '.xlf'
 
-                    self._save_file(local_file, charset, r)
+        # Pull the queued resources
+        if parallel:
+            logger.info("Pulling resources...")
+            perform_parallel_requests()
 
     def push(self, source=False, translations=False, force=False,
-             resources=[], languages=[], skip=False, no_interactive=False,
-             xliff=False, branch=None):
+             resources=None, languages=None, skip=False, no_interactive=False,
+             xliff=False, branch=None, parallel=False):
         """Push all the resources"""
+        languages = languages or []
+        resources = resources or []
+
         resource_list = self.get_chosen_resources(resources)
         self.skip = skip
         self.force = force
@@ -602,6 +640,8 @@ class Project(object):
                                resource=resource_slug)
 
             message = "Pushing resource {}".format(resource)
+            if parallel:
+                message = "Resource {} queued for push".format(resource)
             if branch:
                 message += " for branch {}".format(branch)
             logger.info(message)
@@ -627,7 +667,10 @@ class Project(object):
                     continue
                 # Push source file
                 try:
-                    logger.info("Pushing source file (%s)" % sfile)
+                    push_msg = "Pushing source file (%s)"
+                    if parallel:
+                        push_msg = "Will queue source file (%s) for push"
+                    logger.info(push_msg % sfile)
                     if not self._resource_exists(stats):
                         logger.info("Resource does not exist.  Creating...")
                         fileinfo = "%s;%s" % (resource_slug, slang)
@@ -642,6 +685,7 @@ class Project(object):
                                 self.get_full_path(sfile)
                                 )],
                         params=params,
+                        parallel=parallel,
                     )
                 except Exception as e:
                     if isinstance(e, SSLError):
@@ -673,10 +717,10 @@ class Project(object):
                     push_languages = list(files.keys())
                 else:
                     push_languages = []
-                    for l in languages:
-                        if l in list(lang_map.keys()):
-                            l = lang_map[l]
-                        push_languages.append(l)
+                    for lang in languages:
+                        push_languages.append(lang_map[lang]
+                                              if lang in list(lang_map.keys())
+                                              else lang)
 
                 logger.debug("Languages to push are %s" % push_languages)
 
@@ -709,6 +753,8 @@ class Project(object):
                         continue
 
                     msg = "Pushing '%s' translations (file: %s)"
+                    if parallel:
+                        msg = "Queueing '%s' translations for push (file: %s)"
                     logger.warning(
                         msg % (utils.color_text(remote_lang, "RED"),
                                local_file)
@@ -720,6 +766,7 @@ class Project(object):
                                     self.get_full_path(local_file)
                                     )], language=remote_lang,
                             params=params,
+                            parallel=parallel,
                         )
                         logger.debug("Translation %s pushed." % remote_lang)
                     except utils.HttpNotFound:
@@ -746,8 +793,16 @@ class Project(object):
                         else:
                             logger.error(e)
 
-    def delete(self, resources=[], languages=[], skip=False, force=False):
+        # Push the queued resources
+        if parallel:
+            logger.info("Pushing resources...")
+            perform_parallel_requests()
+
+    def delete(self, resources=None, languages=None, skip=False, force=False):
         """Delete translations."""
+        languages = languages or []
+        resources = resources or []
+
         resource_list = self.get_chosen_resources(resources)
         self.skip = skip
         self.force = force
@@ -865,12 +920,18 @@ class Project(object):
                 raise
 
     def do_url_request(self, api_call, multipart=False, data=None,
-                       files=[], method="GET", skip_decode=False,
-                       params={}, **kwargs):
+                       files=None, method="GET", skip_decode=False,
+                       params=None, parallel=False, no_interactive=False,
+                       **kwargs):
         """Issues a url request."""
+        files = files or []
+        params = params or {}
+
         # Read the credentials from the config file (.transifexrc)
         host = self.url_info['host']
-        username, passwd = self.getset_host_credentials(host)
+        username, passwd = self.getset_host_credentials(
+            host, no_interactive=no_interactive
+        )
         try:
             hostname = self.txrc.get(host, 'hostname')
         except configparser.NoSectionError:
@@ -910,9 +971,19 @@ class Project(object):
                 # in case of PUT we add xliff option as form data
                 if method == 'PUT':
                     data.update(params)
+
+        # Prepare the callback function and arguments
+        cb = kwargs.get("callback", None)
+        args = kwargs.get("callback_args", {})
+
+        if parallel:
+            return utils.queue_request(method, hostname, url, username, passwd,
+                                       data, skip_decode=skip_decode,
+                                       callback=cb, callback_args=args)
+
         return utils.make_request(
             method, hostname, url, username, passwd, data,
-            skip_decode=skip_decode
+            skip_decode=skip_decode, callback=cb, callback_args=args
         )
 
     def _should_update_translation(self, lang, stats, local_file, force=False,
@@ -1160,10 +1231,12 @@ class Project(object):
                 new_translations.append(lang)
         return set(new_translations)
 
-    def _get_stats_for_resource(self):
+    def _get_stats_for_resource(self, no_interactive=False):
         """Get the statistics information for a resource."""
         try:
-            r, charset = self.do_url_request('resource_stats')
+            r, charset = self.do_url_request(
+                'resource_stats', no_interactive=no_interactive
+            )
             logger.debug("Statistics response is %s" % r)
             stats = utils.parse_json(r)
         except utils.HttpNotFound:
@@ -1203,10 +1276,10 @@ class Project(object):
         return selected_resources
 
     def _languages_to_pull(self, languages, files, lang_map, stats, force):
-        """Get a set of langauges to pull.
+        """Get a set of languages to pull.
 
         Args:
-            languages: A list of languages the user selected in cmd.
+            languages: A list of languages the user selected in command.
             files: A dictionary of current local translation files.
         Returns:
             A tuple of a set of existing languages and new translations.
@@ -1231,9 +1304,9 @@ class Project(object):
                     if self._should_add_translation(l, stats, force):
                         new_translations.append(l)
                 else:
-                    if l in list(lang_map.keys()):
-                        l = lang_map[l]
-                    pull_languages.append(l)
+                    pull_languages.append(lang_map[l]
+                                          if l in list(lang_map.keys())
+                                          else l)
             return (set(pull_languages), set(new_translations))
 
     def _extension_for(self, i18n_type):
@@ -1274,9 +1347,8 @@ class Project(object):
 
         host = self.url_info['host']
         try:
-            username = self.txrc.get(host, 'username')
-            passwd = self.txrc.get(host, 'password')
             hostname = self.txrc.get(host, 'hostname')
+            username, passwd = self.getset_host_credentials(host)
         except configparser.NoSectionError:
             raise TransifexrcConfigFileError(
                 "No user credentials found for host %s. Edit "
@@ -1313,7 +1385,7 @@ class Project(object):
         return r
 
     def _get_url_by_pull_mode(self, mode):
-        """Get the url by the pull mode.
+        """Get the URL by the pull mode.
 
         If the pull mode is not valid, the default pull mode will be used.
         """
@@ -1329,7 +1401,7 @@ class Project(object):
         return DEFAULT_PULL_URL
 
     def _get_option(self, resource, option):
-        """Get the value for the option in the config file.
+        """Get the value for the option in the configuration file.
 
         If the option is not in the resource section, look for it in
         the project.
@@ -1359,7 +1431,7 @@ class Project(object):
         self._set_resource_option(resources, key='mode', value=mode)
 
     def _set_resource_option(self, resources, key, value):
-        """Set options in the config file.
+        """Set options in the configuration file.
 
         If resources is empty. set the option globally.
         """
@@ -1370,13 +1442,13 @@ class Project(object):
             self.config.set(r, key, value)
 
     @staticmethod
-    def _save_file(local_file, charset, file_content):
+    def _save_file(local_file="", charset=None, data=None):
         base_dir = os.path.split(local_file)[0]
         utils.mkdir_p(base_dir)
         fd = open(local_file, 'wb')
         if charset is not None:
-            file_content = file_content.encode(charset)
-        fd.write(file_content)
+            data = data.encode(charset)
+        fd.write(data)
         fd.close()
 
     def _set_url_info(self, host, project, resource):
